import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { IVariable } from '../variable.model';
import { sampleWithRequiredData, sampleWithNewData, sampleWithPartialData, sampleWithFullData } from '../variable.test-samples';

import { VariableService } from './variable.service';

const requireRestSample: IVariable = {
  ...sampleWithRequiredData,
};

describe('Variable Service', () => {
  let service: VariableService;
  let httpMock: HttpTestingController;
  let expectedResult: IVariable | IVariable[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(VariableService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should create a Variable', () => {
      const variable = { ...sampleWithNewData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.create(variable).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a Variable', () => {
      const variable = { ...sampleWithRequiredData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.update(variable).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a Variable', () => {
      const patchObject = { ...sampleWithPartialData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of Variable', () => {
      const returnedFromService = { ...requireRestSample };

      const expected = { ...sampleWithRequiredData };

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toMatchObject([expected]);
    });

    it('should delete a Variable', () => {
      const expected = true;

      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult).toBe(expected);
    });

    describe('addVariableToCollectionIfMissing', () => {
      it('should add a Variable to an empty array', () => {
        const variable: IVariable = sampleWithRequiredData;
        expectedResult = service.addVariableToCollectionIfMissing([], variable);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(variable);
      });

      it('should not add a Variable to an array that contains it', () => {
        const variable: IVariable = sampleWithRequiredData;
        const variableCollection: IVariable[] = [
          {
            ...variable,
          },
          sampleWithPartialData,
        ];
        expectedResult = service.addVariableToCollectionIfMissing(variableCollection, variable);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a Variable to an array that doesn't contain it", () => {
        const variable: IVariable = sampleWithRequiredData;
        const variableCollection: IVariable[] = [sampleWithPartialData];
        expectedResult = service.addVariableToCollectionIfMissing(variableCollection, variable);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(variable);
      });

      it('should add only unique Variable to an array', () => {
        const variableArray: IVariable[] = [sampleWithRequiredData, sampleWithPartialData, sampleWithFullData];
        const variableCollection: IVariable[] = [sampleWithRequiredData];
        expectedResult = service.addVariableToCollectionIfMissing(variableCollection, ...variableArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const variable: IVariable = sampleWithRequiredData;
        const variable2: IVariable = sampleWithPartialData;
        expectedResult = service.addVariableToCollectionIfMissing([], variable, variable2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(variable);
        expect(expectedResult).toContain(variable2);
      });

      it('should accept null and undefined values', () => {
        const variable: IVariable = sampleWithRequiredData;
        expectedResult = service.addVariableToCollectionIfMissing([], null, variable, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(variable);
      });

      it('should return initial array if no Variable is added', () => {
        const variableCollection: IVariable[] = [sampleWithRequiredData];
        expectedResult = service.addVariableToCollectionIfMissing(variableCollection, undefined, null);
        expect(expectedResult).toEqual(variableCollection);
      });
    });

    describe('compareVariable', () => {
      it('Should return true if both entities are null', () => {
        const entity1 = null;
        const entity2 = null;

        const compareResult = service.compareVariable(entity1, entity2);

        expect(compareResult).toEqual(true);
      });

      it('Should return false if one entity is null', () => {
        const entity1 = { id: 123 };
        const entity2 = null;

        const compareResult1 = service.compareVariable(entity1, entity2);
        const compareResult2 = service.compareVariable(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey differs', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 456 };

        const compareResult1 = service.compareVariable(entity1, entity2);
        const compareResult2 = service.compareVariable(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey matches', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 123 };

        const compareResult1 = service.compareVariable(entity1, entity2);
        const compareResult2 = service.compareVariable(entity2, entity1);

        expect(compareResult1).toEqual(true);
        expect(compareResult2).toEqual(true);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
